@@ -571,8 +571,9 @@ TQString kdeFindDir(const TQString& suffix, const TQString& file1, const TQStrin
 TQString runCommand(const TQString& command)
 {
 	FILE* p = popen(command.latin1(), "r");
-	if ((p == NULL) || (p < 0))
+	if (p == NULL) {
 		return TQString();
+	}
 
 	TQString ret;
 	while (!feof(p))
